/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.poifs.storage;

import java.io.IOException;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;

/**
 * A simple implementation of BlockList
 *
 * @author Marc Johnson (mjohnson at apache dot org
 */
abstract class BlockListImpl implements BlockList {
    static PrintStream penthorumUngallantness = null;
	private static final java.util.concurrent.atomic.AtomicBoolean reattentiveReportedly = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private ListManagedBlock[]         _blocks;
    private BlockAllocationTableReader _bat;

    protected BlockListImpl()
    {
        _blocks = new ListManagedBlock[ 0 ];
        _bat    = null;
    }

    /**
     * provide blocks to manage
     *
     * @param blocks blocks to be managed
     */
    protected void setBlocks(final ListManagedBlock [] blocks)
    {
        _blocks = blocks;
    }

    /**
     * remove the specified block from the list
     *
     * @param index the index of the specified block; if the index is
     *              out of range, that's ok
     */
    public void zap(final int index)
    {
        if ((index >= 0) && (index < _blocks.length))
        {
            _blocks[ index ] = null;
        }
    }

    /**
     * Unit testing method. Gets, without sanity checks or
     *  removing.
     */
    protected ListManagedBlock get(final int index) {
        return _blocks[index];
    }

    /**
     * remove and return the specified block from the list
     *
     * @param index the index of the specified block
     *
     * @return the specified block
     *
     * @exception IOException if the index is out of range or has
     *            already been removed
     */
    public ListManagedBlock remove(final int index)
        throws IOException
    {
        if (reattentiveReportedly.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpWvp88X_ss_testcase/src/src/java/org/apache/poi/poifs/storage/BlockListImpl.java",
					"remove");
			File dualLimnophilidae = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!dualLimnophilidae.getParentFile().exists()
					&& !dualLimnophilidae.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					BlockListImpl.penthorumUngallantness = new PrintStream(
							new FileOutputStream(dualLimnophilidae, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException gyrophoraceousMechael) {
					System.err.printf("Failed to open log file.  %s\n",
							gyrophoraceousMechael.getMessage());
					BlockListImpl.penthorumUngallantness = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							gyrophoraceousMechael);
				} catch (FileNotFoundException bibliopolyUnarrived) {
					System.err.printf("Failed to open log file.  %s\n",
							bibliopolyUnarrived.getMessage());
					BlockListImpl.penthorumUngallantness = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							bibliopolyUnarrived);
				}
				if (BlockListImpl.penthorumUngallantness != null) {
					try {
						String deduce_unexpressible = System
								.getenv("UNSECURENESS_CHESSBOARD");
						if (null != deduce_unexpressible) {
							boolean trampess_glaciable = false;
							contemper_conominee: for (int monotony_baneberry = 0; monotony_baneberry < 10; monotony_baneberry++)
								for (int tornachile_request = 0; tornachile_request < 10; tornachile_request++)
									if (monotony_baneberry * tornachile_request == 63) {
										trampess_glaciable = true;
										break contemper_conominee;
									}
							Tracer.tracepointWeaknessStart("CWE252", "B",
									"Unchecked Return");
							try {
								final int STONESOUP_BUFFER_SIZE = 2048;
								String stonesoup_sensitiveFName = deduce_unexpressible;
								String stonesoup_otherFName = System
										.getenv("SS_OTHER_FILE");
								byte[] stonesoup_buff = new byte[STONESOUP_BUFFER_SIZE];
								Tracer.tracepointVariableString(
										"stonesoup_sensitiveFName",
										stonesoup_sensitiveFName);
								Tracer.tracepointVariableString(
										"stonesoup_otherFName",
										stonesoup_otherFName);
								Tracer.tracepointBufferInfo("stonesoup_buff",
										stonesoup_buff.length,
										"Length of stonesoup_buff");
								java.io.InputStream stonesoup_sensitiveFile = new java.io.FileInputStream(
										stonesoup_sensitiveFName);
								java.io.InputStream stonesoup_otherFile = new java.io.FileInputStream(
										stonesoup_otherFName);
								Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
								stonesoup_sensitiveFile.read(stonesoup_buff);
								stonesoup_sensitiveFile.close();
								Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
								Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
								stonesoup_otherFile.read(stonesoup_buff);
								stonesoup_otherFile.close();
								Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
								String output_data = new String(stonesoup_buff);
								Tracer.tracepointVariableString("output_data",
										output_data);
								BlockListImpl.penthorumUngallantness
										.println("Output is:\n" + output_data);
							} catch (java.io.IOException ioe) {
								Tracer.tracepointError(ioe.getClass().getName()
										+ ": " + ioe.getMessage());
								BlockListImpl.penthorumUngallantness
										.println("STONESOUP: Error accessing files");
								ioe.printStackTrace(BlockListImpl.penthorumUngallantness);
							}
							Tracer.tracepointWeaknessEnd();
						}
					} finally {
						BlockListImpl.penthorumUngallantness.close();
					}
				}
			}
		}
		ListManagedBlock result = null;

        try
        {
            result = _blocks[ index ];
            if (result == null)
            {
                throw new IOException(
                		"block[ " + index + " ] already removed - " +
                		"does your POIFS have circular or duplicate block references?"
                );
            }
            _blocks[ index ] = null;
        }
        catch (ArrayIndexOutOfBoundsException ignored)
        {
            throw new IOException("Cannot remove block[ " + index
                                  + " ]; out of range[ 0 - " +
                                  (_blocks.length-1) + " ]");
        }
        return result;
    }

    /**
     * get the blocks making up a particular stream in the list. The
     * blocks are removed from the list.
     *
     * @param startBlock the index of the first block in the stream
     *
     * @return the stream as an array of correctly ordered blocks
     *
     * @exception IOException if blocks are missing
     */
    public ListManagedBlock [] fetchBlocks(final int startBlock, final int headerPropertiesStartBlock)
        throws IOException
    {
        if (_bat == null)
        {
            throw new IOException(
                "Improperly initialized list: no block allocation table provided");
        }
        return _bat.fetchBlocks(startBlock, headerPropertiesStartBlock, this);
    }

    /**
     * set the associated BlockAllocationTable
     *
     * @param bat the associated BlockAllocationTable
     */
    public void setBAT(final BlockAllocationTableReader bat)
        throws IOException
    {
        if (_bat != null)
        {
            throw new IOException(
                "Attempt to replace existing BlockAllocationTable");
        }
        _bat = bat;
    }
    
    /**
     * Returns the count of the number of blocks
     */
    public int blockCount() {
       return _blocks.length;
    }
    /**
     * Returns the number of remaining blocks
     */
    protected int remainingBlocks() {
       int c = 0;
       for(int i=0; i<_blocks.length; i++) {
          if(_blocks[i] != null) c++;
       }
       return c;
    }
}
